      program thcomp
      Implicit none
      
      Real x(10000),sptot(10000),sptot_z(10000),spinp(10000)
      Real kTe, tau, z_red, F, E
      Real delta,xmin_in,xmax_in,xmin_out,xmax_out
      character d
      Integer i,j,jl,jmax,ne,io
      character*100 seedf,file_out
      Real ein(10000),dein(10000),fluxin(10000),norm
      
      open(unit=12,file='th.inp')
      read(12,*)d, delta
      read(12,*)d, xmin_out
      read(12,*)d, xmax_out
      read(12,*)d, kTe
      read(12,*)d, tau
      read(12,*)d, z_red
      read(12,*)d, seedf
      read(12,*)file_out
      close(12)

      ne = 0
      norm=0.0
      open(unit=13,file=trim(seedf))
      read(13,*)d
      read(13,*)d
      read(13,*)d
      i=1
      DO
         READ(13,*,iostat=io)ein(i),dein(i),fluxin(i)
         IF (io/=0) EXIT
         fluxin(i)=fluxin(i)/ein(i)/ein(i)
         ein(i)=ein(i)/511
         dein(i)=dein(i)/511
         norm=norm+fluxin(i)*2*dein(i)
         ne=ne+1
         i=i+1
      ENDDO
      close(13)

      xmin_in=ein(1)*511
      xmax_in=ein(ne)*511

      write(*,*)"Generating thcomp spectra for seed file: ",seedf
      write(*,*)" "
      write(*,*)"Seed energy array parameters:"
      write(*,*)"emin_in=",xmin_in
      write(*,*)"emax_in=",xmax_in
      write(*,*)"Output energy array parameters:"
      write(*,*)"delta=",delta
      write(*,*)"emin_out=",xmin_out
      write(*,*)"emax_out=",xmax_out
      write(*,*)" "
      write(*,*)"Spectrum parameters:"
      write(*,*)"kTe=",kTe
      write(*,*)"tau=",tau
      write(*,*)"z=",z_red
      write(*,*)"Output file=",file_out

      do i=1,ne
         fluxin(i)=fluxin(i)/norm !renormalize to 1
      enddo

      call thcompton(ein,fluxin,kTe/511,tau,x,jmax,sptot,spinp,
     >     xmin_in,xmax_in,xmin_out,xmax_out,delta,ne)

      do i = 1, jmax
         sptot_z(i)=0.0
      enddo

      IF(z_red.gt.0.0) THEN
         j = 1
         do i=0,jmax
            do while ((j .lt. ne).and.x(j) .lt. x(i)*(1+z_red))
               j = j + 1
            end do
            if (j .lt. jmax) then
               if (j .gt. 1) then
                  jl = j - 1
                  sptot_z(i) = sptot(jl)+(x(i)*(1+z_red)-x(jl))*
     >                 (sptot(jl+1)-sptot(jl))/(x(jl+1)-x(jl))
               else
                  sptot_z(i)=sptot(1)
               endif
            else
               sptot_z(i)=sptot(j)
            endif
         end do

      ELSE
         sptot_z=sptot
      ENDIF

      open(unit = 14, file=file_out)
      do i = 1, jmax
        E = x(i)*511
        F = sptot_z(i)*511
        write(14, *) E, F
      enddo
      close(unit=14)
      write(*,*)"All done."

      stop
      end

      subroutine thcompton(ein,fluxin,theta0,tau,x,jmax,sptot,spinp,
     >     xmin_in,xmax_in,xmin_out,xmax_out,delta,ne)
      
c  version: January 96
c
c
c     Thermal Comptonization; solves Kompaneets eq. with some
c     relativistic corrections. See Lightman & Zdziarski (1987), ApJ
c     The seed spectrum is blackbody.
      IMPLICIT NONE
      real delta,deltal,xnr,xr,taukn,flz,pi
      real xmin_in,xmax_in,xmin_out,xmax_out
      real sptot(10000),x(10000),dphesc(10000),dphdot(10000),
     >     spinp(10000),rel(10000),bet(10000),c0(10000),c1(10000),
     >     c2(10000)
      real ein(10000),fluxin(10000)
      integer i,j,jmax,jmaxth,jnr,jrel,ne
      real*8 w,w1,z1,z2,z3,z4,z5,z6
      real tau,alpha,theta0,tauthin,tauthick,escape
c  input parameters:
      real theta
      real fnorm,factor

      pi=3.1415927 
c clear arrays (important for repeated calls)
      do 31 j=1,10000
        dphesc(j)=0
        dphdot(j)=0
        rel(j)=0
        bet(j)=0
	c0(j)=0
	c1(j)=0
        c2(j)=0
        sptot(j)=0
        spinp(j)=0
  31  continue

c	a phenomenological relativistic correction to the temperature
c	an increase of internal T at higher values
	theta=theta0*(1+1*theta0+3*theta0**2)
	write(6,*)'internal kT=',theta*511

C
C JMAX - # OF PHOTON ENERGIES
C  
c delta is the 10-log interval of the photon array.
!      delta=0.02 now it is argument
      deltal=delta*log(10.)
!     xmin=1e-4*tempbb
!     xmin=1e-4/511.
      xmin_in=xmin_in/511.
      xmax_in=xmax_in/511.
      xmin_out=xmin_out/511.
      xmax_out=xmax_out/511.

      jmax=int(min(9999.,log10(xmax_out/xmin_out)/delta+1))
C X - ARRAY FOR PHOTON ENERGIES
C
      do 4 j=1,jmax+1
         x(j)=xmin_out*10.**((j-1)*delta)
  4   continue
c compute c0(x), and rel(x) arrays
c c2(x) is the relativistic correction to Kompaneets equation
c rel(x) is the Klein-Nishina cross section divided by the Thomson one
      do 500 j=1,jmax
        w=x(j)
c c2 is the correction to the diffusion coefficient calculated at w1
c w1 is x(j+1/2) (x(i) defined up to jmax+1)
c
        w1=sqrt(x(j)*x(j+1))
        c0(j)= real(w1**4/(1+4.6*w1+1.1*w1**2))
        c1(j)= theta
	c2(j)= 1
        if (w.le.0.05) then
c use asymptotic limit for rel(x) for x less than 0.05
          rel(j)=real(1-2*w+26*w*w/5)
        else
          z1=(1+w)/w**3
          z2=1+2*w
          z3=log(z2)
          z4=2*w*(1+w)/z2
          z5=z3/2/w
          z6=(1+3*w)/z2/z2
          rel(j)=real(0.75*(z1*(z4-z3)+z5-z6))
        end if
 500  continue

c the thermal emision spectrum 
!     jmaxth=int(min(10000.,log10(50*tempbb/xmin)/delta))
!!!!  jmaxth=int(min(10000.,log10(xmax_in/xmin_in)/delta)+1)
      jmaxth=int(min(10000.,real(ne)))
!        planck=15/(pi*tempbb)**4
!        do 5 j=1,jmaxth
!          dphdot(j)=planck*x(j)**2/(exp(x(j)/tempbb)-1)
!     5     continue

      i=1
      do while(x(i).lt.xmin_in)
         dphdot(i)=0.0
         i=i+1
      enddo
      j=1

      do while(x(i).le.xmax_in)
         do while((x(i).lt.ein(j+1)).or.(x(i).eq.xmax_in))
            IF(x(i).eq.xmax_in)THEN
               factor=(x(i)-ein(jmaxth-1))/(ein(jmaxth)-ein(jmaxth-1))
               dphdot(i)=(1.0-factor)*fluxin(jmaxth-1)+
     >              factor*fluxin(jmaxth)
               i=i+1
               GOTO 20
            ENDIF
            factor=(x(i)-ein(j))/(ein(j+1)-ein(j))
            dphdot(i)=(1.0-factor)*fluxin(j)+factor*fluxin(j+1)
            i=i+1
         enddo
         j=j+1
         if(j.eq.jmaxth)exit
      enddo
 20   CONTINUE

      do while(x(i).le.xmax_out)
         dphdot(i)=0.0
         i=i+1
      enddo


c      do 5 j=1,jmaxth
c         dphdot(j)=x(j)**(-gamma)*exp(-x(j)/efold)
c 5    continue

c      j=1
c      do while((x(j).lt.xmin_in).or.(j.eq.jmaxth))
c         dphdot(j)=0.0
c         j=j+1
c      enddo
c
c      j=jmaxth
c      do while((x(j).gt.xmax_in).or.(j.eq.1))
c         dphdot(j)=0.0
c         j=j-1
c      enddo


c     
c compute beta array, the probalility of escape per Thomson time.
c bet evaluated for spherical geometry and nearly uniform sources.
c Between x=0.1 and 1.0, a function flz modifies beta to allow
c the increasingly large energy change per scattering to gradually
c eliminate spatial diffusion

        jnr=int(log10(0.1/xmin_out)/delta+1)
        jnr=min(jnr,jmax-1)
c	epsilon_R zmienione z 1 na 2, AAZ 30.07.2019
        jrel=int(log10(2/xmin_out)/delta+1)
        jrel=min(jrel,jmax)
        xnr=x(jnr)
        xr=x(jrel)
c	tauthin is a phenomenological factor for the optically-thin part of beta
	tauthin=1.2/(1+1*theta0+5*theta0**2)
c	tauthick is a correction factor for the optically-thick part of beta
c	with a phenomenological dependence on theta0. In the limit of low kT, 
c	this correction factor should be 0.25
	tauthick=0.25/(1+1*theta0+3*theta0**2)
        do j=1, jnr-1
          taukn=tau*rel(j)
          bet(j)= 1/(tau*(tauthin+taukn*tauthick))
        end do
        do j=jnr,jrel
          taukn=tau*rel(j)
          flz=(xr-x(j))/(xr-xnr)
          bet(j)= 1/(tau*(tauthin+taukn*tauthick*flz))
        end do
        do j=jrel+1,jmax
          taukn=tau*rel(j)
          bet(j)=1/(tau*tauthin)
        end do

c      Note that tau in thermlc affects only the overall norm, and it is 
c       actually not needed at all. 
      call thermlc 
     >  (1.,theta,deltal,x,jmax,dphesc,dphdot,bet,c0,c1,c2)
c
c     the unscattered fraction in a uniform sphere with uniform 
c     production of photons 
      escape=(2*tau**2 - 1 + exp(-2*tau)*(2*tau+1))
     > *3/(8*tau**3) 
c	the geometrical average with escape for central production of photons 
	escape=Sqrt(escape*Exp(-tau))
      write(*,*) 'escaped fraction=',escape

       alpha=sqrt(2.25+bet(1)/theta)-1.5
       write(*,*) 'alpha = ',alpha
       
c     the spectrum in E F_E
      do 497 j=1,jmax-1
c       add the unscattered fraction
          dphesc(j)=dphesc(j)+escape*dphdot(j)
          sptot(j)=dphesc(j)*x(j)**2
          spinp(j)=dphdot(j)*x(j)**2
c          write(1,*) x(j), sptot(j)
 497  continue
c     the input spectrum
c      do 498 j=1,jmaxth
c         write(2,*) x(j), dphdot(j)*x(j)**2
c 498  continue

c     normalize the spectrum to unit number of photons.
c     sptot is nu F_nu

      fnorm = 0
      do j = 1, jmax
        fnorm = fnorm + sptot(j)/x(j)
      enddo
      fnorm=fnorm*log(x(2)/x(1))
      do j = 1, jmax
         sptot(j) = sptot(j)/fnorm
!         spinp(j) = spinp(j)/fnorm
      enddo
      
      return  
      end

      subroutine thermlc 
     >  (tau,theta,deltal,x,jmax,dphesc,dphdot,bet,c0,c1,c2)
c This program computes the effects of Comptonization by
c nonrelativistic thermal electrons in a sphere including escape, and
c relativistic corrections up to photon energies of 1 MeV.
c the dimensionless photon energy is x=hv/(m*c*c)
c
c The input parameters and functions are:
c dphdot(x), the photon production rate 
c tau, the Thomson scattering depth
c theta, the temperature in units of m*c*c
c c2(x), and bet(x), the coefficients in the K-equation and the
c   probability of photon escape per Thomson time, respectively,
c   including Klein-Nishina corrections
c The output parameters and functions are:
c dphesc(x), the escaping photon density
      implicit none 
      integer jmax
      real tau,theta,deltal
      real x(10000),dphesc(10000), dphdot(10000),bet(10000),c0(10000)
      integer j,jj
      real a(10000),b(10000),c(10000),c20,w1,w2,t1,t2,t3,x32,aa,
     >     c2(10000)
      real d(10000),alp(10000),g(10000),gam(10000),u(10000),c1(10000)
c u(x) is the dimensionless photon occupation number

      c20=tau/deltal
c
c determine u
c define coefficients going into equation
c a(j)*u(j+1)+b(j)*u(j)+c(j)*u(j-1)=d(j)
      do 1 j=2,jmax-1
        w1=sqrt(x(j)*x(j+1))
        w2=sqrt(x(j-1)*x(j))
c  w1 is x(j+1/2)
c  w2 is x(j-1/2)
        a(j)=-c20*c0(j)*(c1(j)/deltal/w1+0.5*c2(j))
        t1=-c20*c0(j)*(0.5*c2(j)-c1(j)/deltal/w1)
        t2=c20*c0(j-1)*(c1(j-1)/deltal/w2+0.5*c2(j-1))
        t3=x(j)**3*(tau*bet(j))
        b(j)=t1+t2+t3
        c(j)=c20*c0(j-1)*(0.5*c2(j-1)-c1(j-1)/deltal/w2)
        d(j)=x(j)*dphdot(j)
   1  continue

c define constants going into boundary terms
c u(1)=aa*u(2) (zero flux at lowest energy)
c u(jx2) given from region 2 above
      x32=sqrt(x(1)*x(2))
      aa=(theta/deltal/x32+0.5)/(theta/deltal/x32-0.5)
c
c zero flux at the highest energy
      u(jmax)=0
c
c invert tridiagonal matrix
      alp(2)=b(2)+c(2)*aa
      gam(2)=a(2)/alp(2)
      do 2 j=3,jmax-1
        alp(j)=b(j)-c(j)*gam(j-1)
        gam(j)=a(j)/alp(j)
  2   continue
      g(2)=d(2)/alp(2)
      do 3 j=3,jmax-2
        g(j)=(d(j)-c(j)*g(j-1))/alp(j)
  3   continue
      g(jmax-1)=
     > (d(jmax-1)-a(jmax-1)*u(jmax)-c(jmax-1)*g(jmax-2))/alp(jmax-1)
      u(jmax-1)=g(jmax-1)
      do 4 j=3,jmax-1
        jj=jmax+1-j
        u(jj)=g(jj)-gam(jj)*u(jj+1)
  4   continue
      u(1)=aa*u(2)

c compute new value of dph(x) and new value of dphesc(x)
      do 9 j=1,jmax
        dphesc(j)=x(j)*x(j)*u(j)*bet(j)*tau
  9   continue
      return
      end

